-----------------------------------------------------------------------------------------------
-- Client Lua Script for Buffhider
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------

require "Window"

-----------------------------------------------------------------------------------------------
-- Buffhider Module Definition
-----------------------------------------------------------------------------------------------
local Buffhider = {}

-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999

-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function Buffhider:new(o)
   o = o or {}
   setmetatable(o, self)
   self.__index = self

   self.buffWatch = {
      Buff = {
         Player = {},
         Target = {}
      },
      Debuff = {
         Player = {},
         Target = {}
      },
   }

   return o
end

function Buffhider:Init()
   Apollo.RegisterAddon(self)
end

function Buffhider:OnSave(eLevel)
   if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then
      return nil
   end
   local savedata = {}
   savedata["watch"] = self.buffWatch
   for k,v in pairs(self.buffWatch) do
      savedata["watch"][k] = v
   end
   return savedata
end

function Buffhider:OnRestore(eLevel, tData)
   if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then
      return nil
   end
   self.buffWatch = tData["watch"]
   for k,v in pairs(self.buffWatch["Buff"]["Player"]) do
      local name = v['Name']
      self.buffWatch["Buff"]["Player"][name]["Handler"] = function(buff) self:HideBuff(buff) end
   end
   for k,v in pairs(self.buffWatch["Debuff"]["Player"]) do
      local name = v['Name']
      self.buffWatch["Debuff"]["Player"][name]["Handler"] = function(debuff) self:HideDebuff(debuff) end
   end
end

-------------------------------------------------------------------------------------
-- Buffhider OnLoad
-----------------------------------------------------------------------------------------------
function Buffhider:OnLoad()
   -- load our form file
   self.xmlDoc = XmlDoc.CreateFromFile("Buffhider.xml")
   self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- Buffhider OnDocLoaded
-----------------------------------------------------------------------------------------------
function Buffhider:OnDocLoaded()

   if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
      self.wndMain = Apollo.LoadForm(self.xmlDoc, "BuffhiderForm", nil, self)
      if self.wndMain == nil then
         Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
         return
      end

      self.wndMain:Show(false, true)

      -- if the xmlDoc is no longer needed, you should set it to nil
      -- self.xmlDoc = nil

      -- Register handlers for events, slash commands and timer, etc.
      -- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
      Apollo.RegisterSlashCommand("bh", "OnBuffhiderOn", self)

      Apollo.RegisterTimerHandler("Buffhider_BuffTimer", "OnUpdate", self)
      Apollo.CreateTimer("Buffhider_BuffTimer", 0.1, true)
      -- Do additional Addon initialization here
   end
end

-----------------------------------------------------------------------------------------------
-- Buffhider Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

-- on SlashCommand "/bh"
function Buffhider:OnBuffhiderOn(cmd,args)

   local function stringstartswith(String,Start)
      if String == "" or String == nil then
         return false
      end
      return string.sub(String,1,string.len(Start))==Start
   end

   if args == "list" then
      ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_System, "Hidden buffs:")
      for _,buff in pairs(self.buffWatch["Buff"]["Player"]) do
         ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_System, buff["Name"])
      end

      ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_System, "Hidden debuffs:")
      for _,debuff in pairs(self.buffWatch["Debuff"]["Player"]) do
         ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_System, debuff["Name"])
      end
   elseif stringstartswith(args,"buff ") then
      args = string.sub(args,string.len("buff ")+1)
      if self.buffWatch["Buff"]["Player"][args] == nil then
         self.buffWatch["Buff"]["Player"][args]= {
            Handler = function(buff) self:HideBuff(buff) end,
            Name = args,
         }
      else
         self.buffWatch["Buff"]["Player"][args]= nil
      end
   elseif stringstartswith(args,"debuff ") then
      args = string.sub(args,string.len("debuff ")+1)
      if self.buffWatch["Debuff"]["Player"][args] == nil then
         self.buffWatch["Debuff"]["Player"][args]= {
            Handler = function(debuff) self:HideDebuff(debuff) end,
            Name = args,
         }
      else
         self.buffWatch["Debuff"]["Player"][args]= nil
      end
   else
      ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_System, "Usage:")
      ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_System, '/bh buff BuffName : hide/show BufName')
      ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_System, '/bh debuff DeBuffName : hide/show DeBufName')
      ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_System, '/bh list : show a list of hidden buffs/debuffs')
   end
end

-- on update
function Buffhider:OnUpdate()
   local unitPlayer = GameLib.GetPlayerUnit()

   if unitPlayer ~= nil then
      self:ProcessBuffs(unitPlayer:GetBuffs(), "Player")
   end
end

function Buffhider:ProcessBuffs(buffs, target)
   for idx, buff in pairs(buffs.arBeneficial) do
      if self.buffWatch["Buff"][target][buff.splEffect:GetName()] ~= nil then
         local watcher = self.buffWatch["Buff"][target][buff.splEffect:GetName()]["Handler"]
         watcher(buff)
      end
   end

   for idx, buff in pairs(buffs.arHarmful) do
      if self.buffWatch["Debuff"][target][buff.splEffect:GetName()] ~= nil then
         local watcher = self.buffWatch["Debuff"][target][buff.splEffect:GetName()]["Handler"]
         watcher(buff)
      end
   end
end

function Buffhider:HideBuff(buff)
   local unitPlayer = GameLib.GetPlayerUnit()

   local function hideBuffFromUi(buffs,buff)
      local buffbar = self:GetBuffBar()
      for idx, bufftohide in pairs(buffs) do
         if bufftohide.splEffect:GetName() == buff.splEffect:GetName() and bufftohide.splEffect:GetName() ~= nil then
            if string.sub(buff.splEffect:GetName(), -string.len(" Mount")) == " Mount" then
               for _,barbuff in pairs(buffbar) do
                  if barbuff:GetBuffTooltip(barbuff) ~= nil and string.sub(barbuff.GetBuffTooltip(barbuff), -string.len(" mount.")) == " mount." and barbuff:IsShown() then
                     barbuff:Show(false,true)
                     return
                  end
               end
            else
               local buffindex = idx

               if buffbar[buffindex] ~= nill and buffbar[buffindex]:IsShown() then
                  buffbar[buffindex]:Show(false,true)
               end
            end
         end
      end
   end

   if unitPlayer ~= nil then
      buffs = unitPlayer:GetBuffs()
      if buffs.arBeneficial ~= nil then
         hideBuffFromUi(buffs.arBeneficial,buff)
      end
   end
end

function Buffhider:HideDebuff(debuff)
   local unitPlayer = GameLib.GetPlayerUnit()

   local function hideDebuffFromUi(debuffs,debuff)
      for idx, debufftohide in pairs(debuffs) do
         if debufftohide.splEffect:GetName() == debuff.splEffect:GetName() then
            local debuffindex = idx
            local debuffbar = self:GetDebuffBar()

            if debuffbar[debuffindex] ~= nill and debuffbar[debuffindex]:IsShown() then
               debuffbar[debuffindex]:Show(false,true)
            end
         end
      end
   end

   if unitPlayer ~= nil then
      buffs = unitPlayer:GetBuffs()
      if buffs.arHarmFul ~= nil then
         hideDebuffFromUi(buffs.arHarmful,debuff)
      end
   end
end

function Buffhider:GetBuffBar()
   local buffbar

   local potatoframes = Apollo.GetAddon("PotatoFrames")
   if  potatoframes ~= nil then
      -- Potato ui
      buffbar = potatoframes.tFrames[1]["buffs"]:GetChildren()[1]:GetChildren()[1]:GetChildren()
   else
      -- Stock ui?
      -- Safely dig into the GUI elements
      local addonTargetFrame = Apollo.GetAddon("TargetFrame")
      if addonTargetFrame == nil then return end

      local luaUnitFrame = addonTargetFrame.luaUnitFrame
      if luaUnitFrame == nil then return end

      local wndMainClusterFrame = luaUnitFrame.wndMainClusterFrame
      if wndMainClusterFrame == nil then return end

      local wndBeneBuffBar = wndMainClusterFrame:FindChild("BeneBuffBar")
      if wndBeneBuffBar == nil then return end

      local playerbuffs = wndBeneBuffBar:GetChildren()
      if playerbuffs == nil then return end
      buffbar = playerbuffs
   end
   return buffbar
end

function Buffhider:GetDebuffBar()
   local debuffbar

   local potatoframes = Apollo.GetAddon("PotatoFrames")
   if  potatoframes ~= nil then
      -- Potato ui
      buffbar = potatoframes.tFrames[1]["debuffs"]:GetChildren()[1]:GetChildren()[1]:GetChildren()
   else
      -- Stock ui?
      -- Safely dig into the GUI elements
      local addonTargetFrame = Apollo.GetAddon("TargetFrame")
      if addonTargetFrame == nil then return end

      local luaUnitFrame = addonTargetFrame.luaUnitFrame
      if luaUnitFrame == nil then return end

      local wndMainClusterFrame = luaUnitFrame.wndMainClusterFrame
      if wndMainClusterFrame == nil then return end

      local wndHarmBuffBar = wndMainClusterFrame:FindChild("HarmBuffBar")
      if wndHarmBuffBar == nil then return end

      local playerdebuffs = wndHarmBuffBar:GetChildren()
      if playerdebuffs == nil then return end
      debuffbar = playerdebuffs
   end
   return debuffbar
end
-----------------------------------------------------------------------------------------------
-- BuffhiderForm Functions
-----------------------------------------------------------------------------------------------
-- when the OK button is clicked
function Buffhider:OnOK()
   self.wndMain:Close() -- hide the window
end

-- when the Cancel button is clicked
function Buffhider:OnCancel()
   self.wndMain:Close() -- hide the window
end


-----------------------------------------------------------------------------------------------
-- Buffhider Instance
-----------------------------------------------------------------------------------------------
local BuffhiderInst = Buffhider:new()
BuffhiderInst:Init()

return Buffhider
